import 'package:flutter/material.dart';

import 'Config.dart';

class SignInButton extends StatelessWidget {
  /// Here are the buttons builder which integrate with button builder
  /// and the buttons list.
  ///
  /// The `SignInButton` class already contains general used buttons.
  /// In case of other buttons, user can always use `SignInButtonBuilder`
  /// to build the signin button.
  static bool useProviderColors = false;
  static Color defaultColor = Colors.white;
  static Color defaultBackgroundColor = Colors.grey.shade800;

  /// onPressed function should be passed in as a required field.
  final VoidCallback onPressed;

  /// button should be used from the enum class `Buttons`
  final Buttons button;

  /// mini is a boolean field which specify whether to use a square mini button.
  final bool mini;

  /// shape is to specify the custom shape of the widget.
  final ShapeBorder? shape;

  /// overrides the default button text
  late final String? text;

  /// overrides the default button padding
  late final EdgeInsets padding;

  // overrides the default button elevation
  late final double elevation;

  late final Color? textColor;
  late final Color? iconColor;
  late final Color? backgroundColor;
  late final Color? splashColor;
  late final Color? highlightColor;
  late final double fontSize;
  late final double iconSize;

  /// the height of the button
  final double? height;

  /// width is default to be 1/1.5 of the screen
  final double? width;

  late final ButtonDescriptor buttonDescriptor;

  /// The constructor is fairly self-explanatory.
  SignInButton(
    this.button, {
    required this.onPressed,
    this.mini = false,
    EdgeInsets? padding,
    this.shape,
    String? text,
    double? elevation,
    double? iconSize,
    double? fontSize,
    Color? backgroundColor,
    Color? textColor,
    Color? iconColor,
    Color? splashColor,
    Color? highlightColor,
    this.height,
    this.width,
  }) {
    this.padding = padding ?? EdgeInsets.zero;
    this.elevation = elevation ?? 1.0;
    this.iconSize = iconSize ?? 14.0;
    this.fontSize = fontSize ?? 14.0;

    buttonDescriptor = buttonConfig.firstWhere((element) => element.type == button);

    this.textColor = textColor ?? defaultColor;
    this.iconColor = iconColor ?? defaultColor;
    this.splashColor = splashColor ?? defaultColor.withOpacity(0.3);
    this.backgroundColor =
        backgroundColor ?? (useProviderColors == true ? buttonDescriptor.backgroundColor : defaultBackgroundColor);
    this.highlightColor = highlightColor ?? this.backgroundColor!.withOpacity(0.3);
    this.text = text ?? buttonDescriptor.text;
  }

  factory SignInButton.mini(
    Buttons button, {
    required VoidCallback onPressed,
    EdgeInsets? padding,
    ShapeBorder? shape,
    String? text,
    double? elevation,
    double? iconSize,
    double? fontSize,
    Color? backgroundColor,
    Color? textColor,
    Color? iconColor,
    Color? splashColor,
    Color? highlightColor,
    double? height,
    double? width,
  }) {
    return SignInButton(
      button,
      onPressed: onPressed,
      backgroundColor: backgroundColor,
      elevation: elevation,
      fontSize: fontSize,
      height: height,
      highlightColor: highlightColor,
      iconColor: iconColor,
      iconSize: iconSize,
      mini: true,
      padding: padding,
      shape: shape,
      splashColor: splashColor,
      text: text,
      textColor: textColor,
      width: width,
    );
  }

  @override
  Widget build(BuildContext context) {
    return MaterialButton(
      key: key,
      minWidth: mini ? (width ?? 35.0) : null,
      height: height,
      elevation: elevation,
      padding: padding,
      color: backgroundColor,
      onPressed: onPressed,
      splashColor: splashColor,
      highlightColor: highlightColor,
      child: _getButtonChild(text: text, icon: buttonDescriptor.icon),
      shape: shape ?? ButtonTheme.of(context).shape,
    );
  }

  /// Get the inner content of a button
  Widget _getButtonChild({String? text, IconData? icon, EdgeInsetsGeometry? innerPadding}) {
    if (mini) {
      return Icon(
        icon,
        size: iconSize,
        color: iconColor,
      );
    }

    if (text == null) {
      return IconButton(
        icon: Icon(
          icon,
          color: iconColor,
          size: iconSize,
        ),
        onPressed: onPressed,

      );
    } else {
      return Container(
        constraints: BoxConstraints(
          maxWidth: width ?? 220,
        ),
        child: Center(
          child: Row(
            crossAxisAlignment: CrossAxisAlignment.center,
            children: <Widget>[
              Padding(
                padding: innerPadding ??
                    EdgeInsets.symmetric(
                      horizontal: 8,
                    ),
                child: Icon(
                  icon,
                  size: iconSize,
                  color: iconColor,
                ),
              ),
              Text(
                text,
                style: TextStyle(
                  color: textColor,
                  fontSize: fontSize,
                ),
              ),
            ],
          ),
        ),
      );
    }
  }
}
